
#ifndef QUERY_H
#define QUERY_H

#include <QObject>
#include <QNetworkAccessManager>
#include <QNetworkRequest>

#include <libxml/xmlmemory.h>
#include <libxml/debugXML.h>
#include <libxml/HTMLtree.h>
#include <libxml/xmlIO.h>
#include <libxml/DOCBparser.h>
#include <libxml/xinclude.h>
#include <libxml/catalog.h>
#include <libxslt/xslt.h>
#include <libxslt/xsltInternals.h>
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>

#include "queryconfiguration.h"

class QTimer;

class Query : public QObject {
    Q_OBJECT
public:
    Query();
    ~Query();

    inline void setConfig(const QueryConfiguration & config ) { m_config = config; }
    inline const QueryConfiguration & config() const { return m_config; }

    /**
     * Start the query and display the result in the 
     * QTextBrowser passed to the constructor.
     *
     * The query is repeated according to the current queryInterval.
     *
     * \see queryInterval
     */
    void start();

    /**
     * Execute the query now onetime, independent
     * of the current timer.
     */
    void queryNowOnce();

    inline QString lastTransformedResult() const { return m_lastTransformedResult; }
    inline QString lastWebServiceResult() const { return m_lastWebServiceResult; }

signals:
    void textChanged(const QString & text);
    void error(int errorCode, QString msg);

private slots:
    void replyFinished(QNetworkReply* reply);
    void updateQuery();
    
private:
    QUrl getQueryUrl() const;
    QByteArray xsltTransform(const QByteArray& data, const QString & xslt);
    void reportError(xmlErrorPtr err);

private:
    QNetworkAccessManager* m_manager;
    QTimer* m_timer;

    QueryConfiguration m_config;
    QUrl m_queryUrl;

    QString m_lastTransformedResult;
    QString m_lastWebServiceResult;
};

#endif 
